@@ -4,6 +4,9 @@
 # $Name:  $
 #
 # $Log: sun_set_rise.pl,v $
+# Revision 0.3  2005/06/23 03:23:53  robert
+# *** empty log message ***
+#
 # Revision 0.2  2005/01/16 03:30:58  robert
 # *** empty log message ***
 #
@@ -43,5 +46,5 @@ $Class::Date::DATE_FORMAT = undef;
 
 $rise = $rise + '1D';
 printf "%s\n",         $now;
-printf "set %s %f\n",  $set, ( $set - $now ) / 3600;
-printf "rise %s %f\n", $rise, ( $rise - $now ) / 3600;
+printf " sets at %s in %.1f hours\n",  $set, ( $set - $now ) / 3600;
+printf "rises at %s in %.1f hours\n", $rise, ( $rise - $now ) / 3600;
