@@ -2,6 +2,9 @@
 # $Name:  $
 #
 # $Log: Control.pm,v $
+# Revision 0.11  2005/06/23 03:36:42  robert
+# *** empty log message ***
+#
 # Revision 0.10  2005/04/17 04:21:43  robert
 # Bug fixes:
 #    - Change memory card reset from 0x300: 0x80 0x05 0x00 to
@@ -91,7 +94,7 @@ use DateTime;
 use IO::Socket;
 use IO::Select;
 
-my $ID = q$Id: Control.pm,v 0.10 2005/04/17 04:21:43 robert Exp $;
+my $ID = q$Id: Control.pm,v 0.11 2005/06/23 03:36:42 robert Exp $;
 my $version = join( ' ', ( split ( ' ', $ID ) )[ 1 .. 3 ] );
 $version =~ s/,v\b//;
 $version =~ s/(\S+)$/($1)/;
@@ -772,19 +775,14 @@ sub _load_positions
       $self->dec_drive_position( $ini{save}{dec_drive_position} );
       $self->focus_0_position( $ini{save}{focus_0_position} );
       $self->focus_1_position( $ini{save}{focus_1_position} );
-      #
-      # We delete the position file after reading so that if the program
-      # exits abnormally, we won't pick up an incorrect position next time
-      # around
-      delete $ini{save};
-      $ini{save} = { };
-      (tied %ini)->RewriteConfig;
       }
    }
 
-sub _save_positions
+sub _save_position
    {
    my $self = shift;
+   my $axis = shift;
+   my $save = shift;
 
    my %ini;
 
@@ -799,17 +797,22 @@ sub _save_positions
       (tied %ini)->SetFileName( $self->position_file );
       }
 
-   $ini{save} = { };
-   $ini{save}{ra_position} = $self->ra_current_second
-      if defined $self->ra_current_second && $self->ra_current_second >= 0;
-   $ini{save}{dec_clamp_position} = $self->dec_clamp_position
-      if defined $self->dec_clamp_position && $self->dec_clamp_position >= 0;
-   $ini{save}{dec_drive_position} = $self->dec_drive_position
-      if defined $self->dec_drive_position && $self->dec_drive_position >= 0;
-   $ini{save}{focus_0_position} = $self->focus_0_position
-      if defined $self->focus_0_position && $self->focus_0_position >= 0;
-   $ini{save}{focus_1_position} = $self->focus_1_position
-      if defined $self->focus_1_position && $self->focus_1_position >= 0;
+   my $key = $axis . '_position';
+   my $value = $axis . '_position';
+
+   if ( $axis eq 'ra' )
+      {
+      $value = 'ra_current_second';
+      }
+
+   if ( $save )
+      {
+      $ini{ save }{ $key } = $self->$value;
+      }
+   else
+      {
+      delete $ini{ save }{ $key };
+      }
 
    (tied %ini)->RewriteConfig;
    untie %ini;
@@ -904,9 +907,6 @@ sub DESTROY
       $status &= $self->driver->motor_off( 'FOCUS_1' );
 #      $status &= $self->driver->motor_off( 'DEC_DRIVE' );
 #      $status &= $self->driver->motor_off( 'DEC_CLAMP' );
-      #
-      # Save the positions
-      $self->_save_positions;
 
       logtrc INFO, "\t%5d bad transfers", $self->_bad_transfer;
       logtrc INFO, "\t%5d overexposures", $self->_overexposure;
@@ -978,11 +978,13 @@ sub dec_clamp_close
       $status &= $self->driver->motor_setup( 'DEC_CLAMP', 'IN' );
       $self->_crater( 'dec_clamp_close setup' ) if !$status;
 
+      $self->_save_position( 'dec_clamp', 0 );
       $status &= $self->driver->motor_move( 'DEC_CLAMP', $self->dec_clamp_in );
       $status &= $self->driver->motor_off( 'DEC_CLAMP' );
       $self->_crater( 'dec_clamp_close move' ) if !$status;
 
       $self->dec_clamp_position( 0 );
+      $self->_save_position( 'dec_clamp', 1 );
       #
       # If the dec drive motor is not off, turn it off now that the clamp is on
       $status &= $self->driver->motor_off( 'DEC_DRIVE' )
@@ -1041,11 +1043,14 @@ sub dec_clamp_open
       $status &= $self->driver->motor_setup( 'DEC_CLAMP', 'OUT' );
       $self->_crater( 'dec_clamp_open setup' ) if !$status;
 
-      $status &= $self->driver->motor_move( 'DEC_CLAMP', $self->dec_clamp_out );
+      $self->_save_position( 'dec_clamp', 0 );
+      $status &= $self->driver->motor_move( 'DEC_CLAMP',
+                                            $self->dec_clamp_out );
       $status &= $self->driver->motor_off( 'DEC_CLAMP' );
       $self->_crater( 'dec_clamp_open move' ) if !$status;
 
       $self->dec_clamp_position( $self->dec_clamp_out );
+      $self->_save_position( 'dec_clamp', 1 );
       }
    }
 
@@ -1109,11 +1114,14 @@ sub dec_drive_move_steps
    $status &= $self->driver->motor_setup( 'DEC_DRIVE', $direction );
 #      if $self->dec_drive_last_direction ne $direction;
 
+   $self->_save_position( 'dec_drive', 0 );
    $status &= $self->driver->motor_move( 'DEC_DRIVE', abs $steps );
+
    $self->_crater( 'dec_drive_move_steps' ) if !$status;
 
    $self->dec_drive_last_direction( $direction );
    $self->dec_drive_position( $self->dec_drive_position + $steps );
+   $self->_save_position( 'dec_drive', 1 );
    #
    # Close the clamp
    $self->dec_clamp_close if $clamped && $self->dec_clamp_use;
@@ -1357,6 +1365,7 @@ sub _motor_to_limit
    my $at_limit = $motor . "_at_limit";
    my $move_steps = $motor . "_move_steps";
 
+   $self->_save_position( $motor, 0 );
    $self->set__driving_to_limit;
    #
    # And move out of the limit if necessary
@@ -1370,8 +1379,9 @@ sub _motor_to_limit
    if ( $steps >= $max_steps )
       {
       $self->$position( undef );
+      $self->_save_position( $motor, 0 );
       $self->_crater( $motor
-         . "_to_limit: could not get off limit in $steps steps ($bulk_steps)")
+         . "_to_limit: could not get off limit in $steps steps ($bulk_steps)");
       }
    #
    # And move on in
@@ -1383,9 +1393,13 @@ sub _motor_to_limit
       $self->$move_steps( -$bulk_steps );
       $steps += $bulk_steps;
       }
-   $self->_crater( $motor
-      . "_to_limit: could not find limit in $steps steps ($bulk_steps)" )
-      if $steps >= $self->$range;
+
+   if ( $steps >= $self->$range )
+      {
+      $self->_save_position( $motor, 0 );
+      $self->_crater( $motor
+         . "_to_limit: could not find limit in $steps steps ($bulk_steps)" );
+      }
    #
    # And back out again, 1 step at a time
    $steps = 0;
@@ -1396,9 +1410,13 @@ sub _motor_to_limit
       $self->$move_steps( 1 );
       ++$steps;
       }
-   $self->_crater( $motor
-      . "_to_limit: could not get off limit in $steps steps (1)")
-      if $steps >= $self->$range;
+
+   if ( $steps >= $self->$range )
+      {
+      $self->_save_position( $motor, 0 );
+      $self->_crater( $motor
+         . "_to_limit: could not get off limit in $steps steps (1)");
+      }
    #
    # And back in again, 1 step at a time
    $steps = 0;
@@ -1411,12 +1429,17 @@ sub _motor_to_limit
       ++$backlash;
       }
    --$backlash;
-   $self->_crater( $motor
-      . "_to_limit: could not find limit in $steps steps (1)" )
-      if $steps >= $self->$range;
+
+   if ( $steps >= $self->$range )
+      {
+      $self->_save_position( $motor, 0 );
+      $self->_crater( $motor
+         . "_to_limit: could not find limit in $steps steps (1)" );
+      }
 
    $steps = $self->$position;
    $self->$position( 0 );
+   $self->_save_position( $motor, 1 );
 
    logtrc INFO, "Missed tracking $motor by $steps steps";
    logtrc INFO, "\tbacklash might be $backlash";
@@ -1549,6 +1572,7 @@ sub ra_on
    my $speed = shift || 1;
    my $status = 1;
 
+   $self->_save_position( 'ra', 0 );
    $status &= $self->driver->motor_ra( $speed );
    my $time = time;
    $self->ra_position( $self->ra_position
@@ -1595,6 +1619,9 @@ sub ra_off
    $self->ra_speed( 0 );
    $self->_crater( "ra_off" )
       if !$status;
+   $self->_save_position( 'ra', 1 );
+
+   return $status;
    }
 
 =over
@@ -1642,6 +1669,7 @@ sub ra_to_limit
 
    my $sidereal = 0;
    my $time;
+   $self->_save_position( 'ra', 0 );
    #
    # Log our current position for debugging purposes
    logtrc DEBUG, "RA position at ra_to_limit %.2f", $self->ra_current_second;
@@ -1703,6 +1731,7 @@ sub ra_to_limit
    my $off = $self->ra_position;
 
    $self->ra_position( 0 );
+   $self->_save_position( 'ra', 1 );
 
    return $off;
    }
@@ -1973,6 +2002,7 @@ sub ra_move_second
 
    my $prev_speed = $self->ra_speed;
    my $delay = $final / $speed;
+   $self->_save_position( 'ra', 0 );
    $self->ra_on( ($delay < 0 ? -1 : 1) * $speed );
    $self->delay( abs $delay );
    if ( $prev_speed != 0 )
@@ -1982,6 +2012,7 @@ sub ra_move_second
    else
       {
       $self->ra_off;
+      $self->_save_position( 'ra', 1 );
       }
    }
 
@@ -2427,12 +2458,14 @@ sub _motor_move_steps
    $status &= $self->driver->motor_setup( $motor, $direction );
 #      if $self->$last_direction ne $direction;
    $self->$last_direction( $direction );
+   $self->_save_position( lc $motor, 0 );
    $status &= $self->driver->motor_move( $motor, abs( $steps ) );
    $self->_crater(
       sprintf '%s_move_steps failed the move ', lc $motor )
       if !$status;
 
    $self->$position( $self->$position + $steps );
+   $self->_save_position( lc $motor, 1 );
    return;
    }
 
